@@ -2,10 +2,10 @@
 #
 # Encoding of Arabic: Tim Buckwalter's Notation ##################################### 2003/06/19
 
-# $Id: Buckwalter.pm 143 2006-11-15 01:16:57Z smrz $
-
 package Encode::Arabic::Buckwalter;
 
+our $VERSION = '14.1';
+
 use 5.008;
 
 use strict;
@@ -13,9 +13,6 @@ use warnings;
 
 use Scalar::Util 'blessed';
 
-our $VERSION = do { q $Revision: 143 $ =~ /(\d+)/; sprintf "%4.2f", $1 / 100 };
-
-
 use Encode::Encoding;
 use base 'Encode::Encoding';
 
@@ -66,6 +63,8 @@ sub import {            # perform import as if Encode were used one level before
 
     require Encode;
 
+    push @Encode::ISA, 'Exporter' unless Encode->can('export_to_level');
+
     Encode->export_to_level(1, @_);
 }
 
@@ -75,7 +74,11 @@ sub encode ($$;$) {
 
     $_[1] = '' if $check;                   # needed by in-place edit
 
-    return encoder $text;
+    $text = encoder $text;
+
+    $text = Encode::encode "utf8", $text if Encode::is_utf8($text);
+
+    return $text;
 }
 
 
@@ -84,7 +87,11 @@ sub decode ($$;$) {
 
     $_[1] = '' if $check;                   # needed by in-place edit
 
-    return decoder $text;
+    $text = Encode::decode "utf8", $text unless Encode::is_utf8($text);
+
+    $text = decoder $text;
+
+    return $text;
 }
 
 
@@ -109,7 +116,7 @@ sub enmode ($$;$$) {
 
                 ( $kshd
                     ? ''
-                    : q [\x{640}] ) .
+                    : q [\x{0640}] ) .
                 q [\x{0623}\x{0624}\x{0625}] .
                 q [\x{060C}\x{061B}\x{061F}] .
                 q [\x{0621}\x{0622}\x{0626}-\x{063A}\x{0641}-\x{064A}] .
@@ -120,7 +127,7 @@ sub enmode ($$;$$) {
                 q [\x{064B}-\x{0650}\x{0670}] .
                 q [\x{0652}] .
                 ( $kshd
-                    ? q [\x{640}]
+                    ? q [\x{0640}]
                     : '' )
 
                 ,
@@ -208,12 +215,14 @@ sub demode ($$;$$) {
 
                 ( $kshd
                     ? ''
-                    : q [\x{640}] ) .
+                    : q [\x{0640}] ) .
                 q [\x{0623}\x{0624}\x{0625}] .
                 q [\x{060C}\x{061B}\x{061F}] .
-                q [\x{0621}\x{0622}\x{0626}-\x{063A}\x{0641}-\x{064A}] .
+                q [\x{0621}\x{0622}\x{0626}\x{0627}\x{0628}\x{0629}\x{062A}\x{062B}\x{062C}\x{062D}\x{062E}] .
+                q [\x{062F}\x{0630}\x{0631}\x{0632}\x{0633}\x{0634}\x{0635}\x{0636}\x{0637}\x{0638}\x{0639}] .
+                q [\x{063A}\x{0641}\x{0642}\x{0643}\x{0644}\x{0645}\x{0646}\x{0647}\x{0648}\x{0649}\x{064A}] .
                 q [\x{067E}\x{0686}\x{0698}\x{06A4}\x{06AF}] .
-                q [\x{0660}-\x{0669}] .
+                q [\x{0660}\x{0661}\x{0662}\x{0663}\x{0664}\x{0665}\x{0666}\x{0667}\x{0668}\x{0669}] .
                 ( $mode == 0
                     ? q [\x{0671}]
                     : q [\x{0627}] ) .
@@ -221,7 +230,7 @@ sub demode ($$;$$) {
                     ? ''
                     : q [\x{0651}] . ( $mode == 2
                                     ? ''
-                                    : q [\x{064B}-\x{0650}\x{0670}] . ( $mode == 3
+                                    : q [\x{064B}\x{064C}\x{064D}\x{064E}\x{064F}\x{0650}\x{0670}] . ( $mode == 3
                                                         ? ''
                                                         : q [\x{0652}] ) ) )
 
@@ -235,7 +244,7 @@ sub demode ($$;$$) {
             sub decoder ($) {
 
                 $_[0] =~ tr[/ . $set[0] . q /]
-                           [/ . $set[1] . q /]d;
+                           [/ . $set[1] . q /]/ . (($kshd or $mode > 0) ? 'd' : '') . q /;
 
                 return $_[0];
             }
@@ -256,11 +265,6 @@ __END__
 Encode::Arabic::Buckwalter - Tim Buckwalter's transliteration of Arabic
 
 
-=head1 REVISION
-
-    $Revision: 143 $        $Date: 2006-11-15 02:16:57 +0100 (Wed, 15 Nov 2006) $
-
-
 =head1 SYNOPSIS
 
     use Encode::Arabic::Buckwalter;         # imports just like 'use Encode' would, plus more
@@ -272,7 +276,7 @@ Encode::Arabic::Buckwalter - Tim Buckwalter's transliteration of Arabic
 
     # shell filter of data, e.g. in *n*x systems instead of viewing the Arabic script proper
 
-    % perl -MEncode::Arabic::Buckwalter -pe 'encode "buckwalter", decode "utf8", $_'
+    % perl -MEncode::Arabic::Buckwalter -pe '$_ = encode "buckwalter", decode "utf8", $_'
 
     # employing the modes of conversion for filtering and trimming
 
@@ -362,21 +366,15 @@ Tim Buckwalter's Qamus  L<http://www.qamus.org/>
 Buckwalter Arabic Morphological Analyzer
     L<http://www.ldc.upenn.edu/Catalog/CatalogEntry.jsp?catalogId=LDC2002L49>
 
-Xerox Arabic Home Page  L<http://www.arabic-morphology.com/>
-
 
 =head1 AUTHOR
 
-Otakar Smrz, L<http://ufal.mff.cuni.cz/~smrz/>
-
-    eval { 'E<lt>' . ( join '.', qw 'otakar smrz' ) . "\x40" . ( join '.', qw 'mff cuni cz' ) . 'E<gt>' }
-
-Perl is also designed to make the easy jobs not that easy ;)
+Otakar Smrz C<< <otakar-smrz users.sf.net> >>, L<http://otakar-smrz.users.sf.net/>
 
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003-2006 by Otakar Smrz
+Copyright (C) 2003-2016 Otakar Smrz
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
